@@ -6,19 +6,46 @@ QSDsan: Quantitative Sustainable Design for sanitation and resource recovery sys
 
 This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
-    Joy Cheung <joycheung1994@gmail.com>
+    Joy Zhang <joycheung1994@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
+# Check system environment, Python 3.7 and below will have issues unpickling saved results
+import sys
+py_version = sys.version.split('.')
+_PY_MAJOR, _PY_MINOR = int(py_version[0]), int(py_version[1])
+
+if (_PY_MAJOR, _PY_MINOR) <= (3, 7):
+    from warnings import warn
+    if (_PY_MAJOR, _PY_MINOR) >= (3, 5):
+        try: import pickle5 as _pk
+        except ModuleNotFoundError:
+            warn(f'Python version {_PY_MAJOR}.{_PY_MINOR} does not support Pickle Protocol 5, '
+                 'installing `pickle5` by running `pip install pickle5` in your '
+                 'command/Anaconda prompt or terminal can reduce the loading time.\n'
+                 'For further information, check https://pypi.org/project/pickle5/.')
+            _pk = None
+    else:
+        warn(f'Python version {_PY_MAJOR}.{_PY_MINOR} does not support Pickle Protocol 5, '
+             'and will have slower speed in when loading the default processes.')
+        _pk = None
+    del warn
+else:
+    import pickle as _pk
+
+
 import pkg_resources
 try:
     __version__ = pkg_resources.get_distribution('qsdsan').version
 except pkg_resources.DistributionNotFound:  # pragma: no cover
     __version__ = None
 
+del sys, py_version, pkg_resources
+
+
 import thermosteam as tmo
 import biosteam as bst
 Chemical = tmo.Chemical
@@ -26,10 +53,14 @@ Chemicals = tmo.Chemicals
 CompiledChemicals = tmo.CompiledChemicals
 Stream = tmo.Stream
 set_thermo = tmo.settings.set_thermo
+get_components = tmo.settings.get_chemicals
+get_thermo = tmo.settings.get_thermo
 
+PowerUtility = bst.PowerUtility
 MultiStream = tmo.MultiStream
 Unit = bst.Unit
 System = bst.System
+Model = bst.Model
 Flowsheet = bst.Flowsheet
 CEPCI = bst.CE # Chemical Engineering Plant Cost Index
 CEPCI_by_year = bst.units.design_tools.CEPCI_by_year
@@ -91,5 +122,4 @@ __all__ = (
     *_sanunit.__all__,
     *_simple_tea.__all__,
     *_lca.__all__,
-           )
-
+           )
\ No newline at end of file
