@@ -33,9 +33,20 @@ _checked_properties = tmo._chemical._checked_properties
 lock_phase = tmo._chemical.lock_phase
 display_asfunctor = tmo._chemical.display_asfunctor
 copy_maybe = tmo.utils.copy_maybe
+get_component_data = tmo._chemical.get_chemical_data
 
 
 # %%
+# =============================================================================
+# Filling missing properties
+# =============================================================================
+
+def unpickle_component(cmp_data):
+    cmp = object.__new__(Component)
+    setfield = setattr
+    for field, value in cmp_data.items():
+        setfield(cmp, field, value)
+    return cmp
 
 # =============================================================================
 # Representation
@@ -184,7 +195,7 @@ class Component(Chemical):
 
     Examples
     --------
-    `Component and WasteStream <https://qsdsan.readthedocs.io/en/latest/tutorials/Component_and_WasteStream.html>`_
+    `Component <https://qsdsan.readthedocs.io/en/latest/tutorials/Component.html>`_
 
 
     See Also
@@ -234,6 +245,11 @@ class Component(Chemical):
         self.i_NOD = i_NOD
         return self
 
+
+    def __reduce__(self):
+        return unpickle_component, (get_component_data(self),)
+
+
     def _atom_frac_setter(self, atom=None, frac=None):
         if self.formula:
             if frac:
@@ -300,7 +316,8 @@ class Component(Chemical):
     @property
     def i_mass(self):
         '''[float] Mass content of the component, [g Component/g measure unit].'''
-        return self._i_mass or 1.
+        if self._i_mass is None: return 1
+        else: return self._i_mass
     @i_mass.setter
     def i_mass(self, i):
         if self.atoms:
