@@ -6,7 +6,7 @@ QSDsan: Quantitative Sustainable Design for sanitation and resource recovery sys
 
 This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
-    Joy Cheung <joycheung1994@gmail.com>
+    Joy Zhang <joycheung1994@gmail.com>
 
 Part of this module is based on the Thermosteam package:
 https://github.com/BioSTEAMDevelopmentGroup/thermosteam
@@ -32,6 +32,7 @@ _num_component_properties = _component._num_component_properties
 _key_component_properties = _component._key_component_properties
 # _TMH = tmo.base.thermo_model_handle.ThermoModelHandle
 _PH = tmo.base.phase_handle.PhaseHandle
+DomainError = tmo.exceptions.DomainError
 
 
 # %%
@@ -57,7 +58,7 @@ class Components(Chemicals):
 
     Examples
     --------
-    `Component and WasteStream <https://qsdsan.readthedocs.io/en/latest/tutorials/Component_and_WasteStream.html>`_
+    `Component <https://qsdsan.readthedocs.io/en/latest/tutorials/Component.html>`_
 
     See Also
     --------
@@ -158,7 +159,7 @@ class Components(Chemicals):
         setattr(self, '__class__', CompiledComponents)
         try: self._compile(components, skip_checks)
         except Exception as error:
-            setattr(self, '__class__', Chemicals)
+            setattr(self, '__class__', Components)
             setattr(self, '__dict__', {i.ID: i for i in components})
             raise error
 
@@ -302,11 +303,9 @@ class Components(Chemicals):
 
                 COPY_V = False # if don't have V model, set those to default
                 if isa(i.V, _PH):
-                    if not i.V.l.valid_methods(298.15):
-                        COPY_V = True
+                    if not i.V.l.valid_methods(298.15): COPY_V = True
                 else:
-                    if not i.V.valid_methods(298.15):
-                        COPY_V = True
+                    if not i.V.valid_methods(298.15): COPY_V = True
 
                 if COPY_V:
                     if i.particle_size in ('Soluble', 'Dissolved gas'):
@@ -389,6 +388,10 @@ class CompiledComponents(CompiledChemicals):
     '''
     A subclass of :class:`thermosteam.CompiledChemicals`, contains `Component` objects as attributes.
 
+    Examples
+    --------
+    `Component <https://qsdsan.readthedocs.io/en/latest/tutorials/Component.html>`_
+
     See Also
     --------
     `thermosteam.CompiledChemicals <https://thermosteam.readthedocs.io/en/latest/Chemicals.html>`_
@@ -409,10 +412,13 @@ class CompiledComponents(CompiledChemicals):
             setfield = setattr
             for cmp in components:
                 setfield(self, cmp.ID, cmp)
-            self._compile()
+            self._compile(components)
             cache[components] = self
         return self
 
+    def __reduce__(self):
+        return CompiledComponents, (self.tuple, )
+
 
     def __contains__(self, component):
         if isinstance(component, str):
