@@ -172,7 +172,6 @@ class ImpactItem:
                 try:
                     CF_value, CF_unit = value # unit provided for CF
                     self.add_indicator(indicator, CF_value, CF_unit)
-                # except: breakpoint()
                 except Exception as e:
                     if 'unpack' in str(sys.exc_info()[1]):
                         self.add_indicator(indicator, value)
@@ -266,6 +265,7 @@ class ImpactItem:
                 new.source = self
         else:
             new = copy_attr(new, self, skip=('_ID', '_source'))
+            new.source = None
 
         return new
 
@@ -621,6 +621,7 @@ class StreamImpactItem(ImpactItem):
                 new.source = self
         else:
             new = copy_attr(new, self, skip=('_ID', '_linked_stream', '_source'))
+            new.source = None
 
         if stream:
             stream.stream_impact_item = new
