@@ -6,6 +6,7 @@ QSDsan: Quantitative Sustainable Design for sanitation and resource recovery sys
 
 This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
+    Joy Zhang <joycheung1994@gmail.com>
 
 Part of this module is based on the BioSTEAM package:
 https://github.com/BioSTEAMDevelopmentGroup/biosteam
@@ -18,11 +19,13 @@ for license details.
 
 # %%
 
+import numpy as np
 from collections import defaultdict
 from collections.abc import Iterable
-from biosteam.utils.misc import format_title
+from matplotlib import pyplot as plt
+from biosteam.utils import Inlets, Outlets, format_title
 from . import currency, Unit, Stream, SanStream, WasteStream, \
-    Construction, Transportation
+    Construction, Transportation, System
 
 __all__ = ('SanUnit',)
 
@@ -60,6 +63,10 @@ def _update_init_with(init_with, ins_or_outs, size):
 
     return new_init_with
 
+def _get_inf_state(inf):
+    inf._init_state()
+    return inf._state
+
 add2list = lambda lst, item: lst.extend(item) if isinstance(item, Iterable) \
     else lst.append(item)
 
@@ -113,7 +120,7 @@ class SanUnit(Unit, isabstract=True):
         .. note::
 
             Regardless of `F_BM_default`, design (F_D), pressure (F_P),
-            and material (F_M) factors are all defaulted to 0.
+            and material (F_M) factors are all defaulted to 1.
 
     See Also
     --------
@@ -127,8 +134,7 @@ class SanUnit(Unit, isabstract=True):
     def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
                  construction=(), transportation=(), equipments=(),
                  add_OPEX={}, uptime_ratio=1., lifetime=None, F_BM_default=None,
-                 **kwargs):
-
+                 isdynamic=False, **kwargs):
         self._register(ID)
         self._specification = None
         self._load_thermo(thermo)
@@ -139,22 +145,21 @@ class SanUnit(Unit, isabstract=True):
         self._init_results()
         self._init_specification()
         self._assert_compatible_property_package()
-
         self.construction = construction
         self.transportation = transportation
         for equip in equipments:
             equip._linked_unit = self
         self.equipments = equipments
-
         self.add_OPEX = add_OPEX
         self.uptime_ratio = 1.
         self.lifetime = lifetime
-
         if F_BM_default:
             F_BM = self.F_BM
             self.F_BM = defaultdict(lambda: F_BM_default)
             self.F_BM.update(F_BM)
-
+        self.isdynamic = isdynamic
+        # For units with different states, should update it in the unit's ``__init__``
+        self._state_header = [f'{cmp.ID} [mg/L]' for cmp in self.components] + ['Q [m3/d]']
         for attr, val in kwargs.items():
             setattr(self, attr, val)
 
@@ -199,13 +204,25 @@ class SanUnit(Unit, isabstract=True):
         return converted
 
 
+    def _init_dynamic(self):
+        self._state = None
+        self._dstate = None
+        self._state_header = [f'{cmp.ID} [mg/L]' for cmp in self.components]
+        self._ODE = None
+        self._mock_dyn_sys = System(self.ID+'_dynmock', path=(self,))
+
+
     def _init_ins(self, ins, init_with):
         super()._init_ins(ins)
-        self._ins = self._convert_stream(ins, self.ins, init_with, 'ins')
+        converted = self._convert_stream(ins, self.ins, init_with, 'ins')
+        self._ins = Inlets(self, self._N_ins, converted, self._thermo,
+                           self._ins_size_is_fixed, self._stacklevel)
 
     def _init_outs(self, outs, init_with):
         super()._init_outs(outs)
-        self._outs = self._convert_stream(outs, self.outs, init_with, 'outs')
+        converted = self._convert_stream(outs, self.outs, init_with, 'outs')
+        self._outs = Outlets(self, self._N_outs, converted, self._thermo,
+                           self._outs_size_is_fixed, self._stacklevel)
 
     def _init_results(self):
         super()._init_results()
@@ -253,6 +270,79 @@ class SanUnit(Unit, isabstract=True):
         info = info.replace('\n ', '\n    ')
         return info[:-1]
 
+    def simulate(self, t_span=(0, 0), start_from_cached_state=True, 
+                 solver='', **kwargs):
+        '''
+        Converge mass and energy flows, design, and cost the unit.
+        
+        .. note::
+            
+            If this unit is a dynamic unit, ODEs will be run after ``_run``
+            and/or ``specification``.
+        
+        Parameters
+        ----------
+        t_span : tuple(float, float)
+            Integration time span for dynamic units.
+        start_from_cached_state: bool
+            Whether to start from the cached state.
+        kwargs : dict
+            Other keyword arguments that will be passed to ``scipy.integrate.solve_ivp``
+            
+        See Also
+        --------
+        `scipy.integrate.solve_ivp <https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.solve_ivp.html>`_
+        '''
+        super().simulate()
+        if self.isdynamic:
+            sys = self._mock_dyn_sys
+            sys._feeds = self.ins
+            sys._products = self.outs
+            sys.simulate(t_span=t_span,
+                         start_from_cached_state=start_from_cached_state,
+                         **kwargs)
+            self._summary()
+
+    def plot_state_over_time(self, system=None, state_var=()):
+        '''
+        Plot the selected state variable over simulated time,
+        all state variables will be plotted if not given.
+        
+        Parameters
+        ----------
+        system : obj
+            The ``System`` object where this unit is simulated,
+            leave as None if this unit is simulated alone.
+        state_var : str of Iterable
+            Name of the state variables, if unsure, check the `_state_header` attribute.
+        '''
+        if not self.isdynamic:
+            raise RuntimeError(f'Unit {self.ID} is not a dynamic unit, '
+                               'cannot plot state over time.')
+        state_var = [state_var] if isinstance(state_var, str) else state_var
+        sys = system or self._mock_dyn_sys
+        try:
+            df = sys._state2df()
+        except:
+            system_or_unit = 'system' if sys==system else 'unit'
+            ID = sys.ID.rstrip('_dynmock')
+            raise RuntimeError(f'The {system_or_unit} {ID} has not been simulated, '
+                               'please simulate first.')    
+        t = df.iloc[:, 0]
+        unit_df = df.xs(self.ID, axis=1)
+        variables = [i.split(' ')[0] for i in self._state_header]
+        units = [i.split(' ')[1] for i in self._state_header]
+        unit_df.columns = variables
+        if state_var:
+            unit_df = unit_df.loc[:, state_var]
+        fig, ax = plt.subplots(figsize=(8, 4.5))
+        for var in unit_df.columns:
+            ax.plot(t, getattr(unit_df, var), '-o',
+                    label=f'{var} {units[variables.index(var)]}')
+        ax.legend(loc='best')
+        ylabel = 'Concentration' if 'Q' not in state_var else 'Concentration or flow'
+        ax.set(xlabel='Time [d]', ylabel=ylabel)
+        return fig, ax
 
     def show(self, T=None, P=None, flow='g/hr', composition=None, N=15, IDs=None, stream_info=True):
         '''Print information of the unit, including waste stream-specific information.'''
@@ -294,6 +384,36 @@ class SanUnit(Unit, isabstract=True):
         '''[Components] The :class:`~.Components` object associated with this unit.'''
         return self.chemicals
 
+    @property
+    def isdynamic(self):
+        '''[bool] Whether the unit runs dynamically within a system.'''
+        return self._isdynamic
+    @isdynamic.setter
+    def isdynamic(self, i):
+        if hasattr(self, '_isdynamic'):
+            if self._isdynamic == bool(i):
+                return
+        else:
+            self._isdynamic = bool(i)
+            self._init_dynamic()
+
+
+    def _collect_ins_state(self):
+        return np.array([inf._state for inf in self._ins])
+        # states = []
+        # for inf in self.ins:
+        #     state = inf._state
+        #     if state is not None:
+        #         if not isinstance(state, float):
+        #             states.append(state)
+        #         else:
+        #             inf._init_state()
+        #             states.append(inf._state)
+        # return np.array(states)
+                
+            
+    def _collect_ins_dstate(self):
+        return np.array([inf._dstate for inf in self._ins])
 
     @property
     def construction(self):
@@ -360,7 +480,17 @@ class SanUnit(Unit, isabstract=True):
             results.loc[('Additional OPEX', ''), :] = ('USD/hr', 0)
         else:
             for k, v in self.add_OPEX.items():
-                results.loc[(k, ''), :] = ('USD/hr', v)
+                if not with_units:
+                    results.loc[(k, '')] = v
+                else:
+                    try: results.loc[(k, ''), :] = ('USD/hr', v)
+                    # When `results` is a series instead of dataframe,
+                    # might not need this
+                    except ValueError:
+                        results = results.to_frame(name=self.ID)
+                        results.insert(0, 'Units', '')
+                        results.loc[(k, ''), :] = ('USD/hr', v)
+                        results.columns.name = type(self).__name__
         if with_units:
             results.replace({'USD': f'{currency}', 'USD/hr': f'{currency}/hr'},
                             inplace=True)
@@ -388,17 +518,10 @@ class SanUnit(Unit, isabstract=True):
         Equipment without provided lifetime will be assumed to have the same
         lifetime as the TEA/LCA.
         '''
-        if self._lifetime is not None:
-            return self._lifetime
-        elif self._default_equipment_lifetime:
-            return self._default_equipment_lifetime
-        else:
-            return None
+        return self.equipment_lifetime
     @lifetime.setter
     def lifetime(self, i):
         if not i:
-            self._lifetime = None
-        elif isinstance(i, dict):
-            self._default_equipment_lifetime.update(i)
+            self.equipment_lifetime = {}
         else:
-            self._lifetime = int(i)
\ No newline at end of file
+            self.equipment_lifetime = i if isinstance(i, dict) else int(i)
\ No newline at end of file
