@@ -5,7 +5,7 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Joy Cheung <joycheung1994@gmail.com>
+    Joy Zhang <joycheung1994@gmail.com>
     Yalin Li <zoe.yalin.li@gmail.com>
 
 Part of this module is based on the Thermosteam package:
@@ -26,13 +26,14 @@ TODO:
 # %%
 
 import numpy as np
-from thermosteam import settings
-from . import Components, Stream, MultiStream, SanStream, MissingSanStream, \
-    set_thermo
+import flexsolve as flx
+from free_properties import PropertyFactory, property_array
+from thermosteam import settings, indexer
+from . import SanStream, MissingSanStream
 from .utils import auom, copy_attr
 from warnings import warn
 
-__all__ = ('WasteStream', 'MissingWasteStream')
+__all__ = ('WasteStream', 'MissingWasteStream', 'get_mock_conc')
 
 
 _defined_composite_vars = ('COD', 'BOD5', 'BOD', 'uBOD', 'NOD', 'ThOD', 'cnBOD',
@@ -43,7 +44,8 @@ _common_composite_vars = ('_COD', '_BOD', '_uBOD', '_TC', '_TOC', '_TN',
                           '_dry_mass', '_charge', '_ThOD', '_cnBOD')
 
 _ws_specific_slots = (*_common_composite_vars,
-                      '_pH', '_SAlk', '_ratios', '_stream_impact_item')
+                      '_pH', '_SAlk', '_ratios', '_stream_impact_item', 
+                      '_state', '_dstate')
 
 _specific_groups = {'S_VFA': ('S_Ac', 'S_Prop'),
                     'X_Stor': ('X_OHO_PHA', 'X_GAO_PHA', 'X_PAO_PHA',
@@ -74,7 +76,8 @@ conc_unit = auom('mg/L')
 # Util functions
 # =============================================================================
 
-_load_components = settings.get_default_chemicals
+_load_components = settings.get_chemicals
+_load_thermo = settings.get_thermo
 
 def to_float(stream, slot):
     return 0. if getattr(stream, slot) is None else float(getattr(stream, slot))
@@ -88,7 +91,7 @@ def _calib_SF_iN(components, concentrations, STKN):
     other_stkn = SN - SF_N - SNOx_N
     SF_N = STKN - other_stkn
     if SF_N < 0:
-        raise ValueError("Negative N content for S_F was estimated.")
+        raise ValueError("Negative N content {SF_N} for S_F was estimated.")
     return SF_N/concentrations['S_F']
 
 def _calib_XBsub_iN(components, concentrations, XTKN):
@@ -96,7 +99,7 @@ def _calib_XBsub_iN(components, concentrations, XTKN):
     other_xtkn = (cmp_c * components.i_N * components.x).sum() - concentrations['X_B_Subst'] * components.X_B_Subst.i_N
     XB_Subst_N = XTKN - other_xtkn
     if XB_Subst_N < 0:
-        raise ValueError("Negative N content for X_B_Subst was estimated.")
+        raise ValueError(f"Negative N content {XB_Subst_N} for X_B_Subst was estimated.")
     return XB_Subst_N/concentrations['X_B_Subst']
 
 
@@ -105,7 +108,7 @@ def _calib_XBsub_iP(components, concentrations, TP):
     other_p = (cmp_c * components.i_P).sum() - concentrations['X_B_Subst'] * components.X_B_Subst.i_P
     XB_Subst_P = TP - other_p
     if XB_Subst_P < 0:
-        raise ValueError("Negative P content for X_B_Subst was estimated.")
+        raise ValueError(f"Negative P content {XB_Subst_P} for X_B_Subst was estimated.")
     return XB_Subst_P/concentrations['X_B_Subst']
 
 def _calib_XBsub_fBODCOD(components, concentrations, substrate_IDs, BOD):
@@ -115,10 +118,91 @@ def _calib_XBsub_fBODCOD(components, concentrations, substrate_IDs, BOD):
     other_BOD = (cmp_c * (components.x + components.c + components.s) * components.f_BOD5_COD).sum() - (c_sub * XB_sub.f_BOD5_COD).sum()
     fbodtocod_sub = (BOD - other_BOD)/c_sub.sum()
     if fbodtocod_sub > 1 or fbodtocod_sub < 0:
-        raise ValueError("BOD5-to-COD ratio for X_B_Subst and X_Stor was estimated out of range [0,1].")
+        raise ValueError(f"BOD5-to-COD ratio {fbodtocod_sub} for X_B_Subst and X_Stor was estimated out of range [0,1].")
     return fbodtocod_sub
 
 
+# Indexer for nicer display
+@property
+def group_conc_compositions(self):
+    raise AttributeError('cannot set group by concentratino')
+
+ComponentConcentrationIndexer, ConcentrationIndexer = \
+    indexer._new_Indexer('Concentration', 'mg/L', group_conc_compositions)
+
+ChemicalMolarFlowIndexer = indexer.ChemicalMolarFlowIndexer
+
+@PropertyFactory(slots=('name', 'mol', 'index', 'F_vol', 'MW',
+                        'phase', 'phase_container'))
+def ConcentrationProperty(self):
+    '''Concentration flows, in mg/L (g/m3).'''
+    f_mass = self.mol[self.index] * self.MW
+    phase = self.phase or self.phase_container.phase
+    if phase != 'l':
+        raise AttributeError('Concentration only valid for liquid phase.')
+    V_sum = self.F_vol
+    # if V_sum==0:
+    #     raise RuntimeError('WasteStream is empty, concentration cannot be calculated.')
+    return 1000. * f_mass / V_sum if f_mass else 0.
+@ConcentrationProperty.setter
+def ConcentrationProperty(self, value):
+    raise AttributeError('Cannot set flow rate by concentration.')
+
+def by_conc(self, TP):
+    '''
+    Return a ComponentConcentrationIndexer that references this object's
+    molar and volume data (volume relies on molar).
+
+    Parameters
+    ----------
+    TP : ThermalCondition
+
+    '''
+    try:
+        conc = self._data_cache[TP]
+    except:
+        cmps = self.chemicals
+        mol = self.data
+        F_vol = self.by_volume(TP).data.sum()
+        conc = np.zeros_like(mol, dtype=object)
+        for i, cmp in enumerate(cmps):
+            conc[i] = ConcentrationProperty(cmp.ID, mol, i, F_vol, cmp.MW,
+                                            None, self._phase)
+        self._data_cache[TP] = \
+        conc = ComponentConcentrationIndexer.from_data(property_array(conc),
+                                                      self._phase, cmps,
+                                                      False)
+    return conc
+indexer.ChemicalMolarFlowIndexer.by_conc = by_conc
+
+def by_conc(self, TP):
+    '''
+    Raise an error for attempt multi-phase usage
+    as concentration only valid for liquid).
+    '''
+    raise AttributeError('Concentration only valid for liquid phase.')
+
+indexer.MolarFlowIndexer.by_conc = by_conc; del by_conc
+
+@PropertyFactory(units='mg/L')
+def MockConcentrationProperty(self):
+    '''Mock array to show concentration flows nicely, in mg/L (g/m3).'''
+    return self.data
+@MockConcentrationProperty.setter
+def MockConcentrationProperty(self, value):
+    self.data = value
+
+def get_mock_conc(IDs, data={}):
+    '''
+    Generate a MockConcentrationProperty based on the given IDs and data,
+    concentration of IDs not included in data will be default to 0.
+    '''
+    dct = {ID: 0. for ID in IDs}
+    dct.update(data)
+    return property_array([MockConcentrationProperty(k, v) for k, v in dct.items()])
+    
+del PropertyFactory
+
 
 # %%
 
@@ -186,7 +270,7 @@ class WasteStream(SanStream):
 
     Examples
     --------
-    `Component and WasteStream <https://qsdsan.readthedocs.io/en/latest/tutorials/Component_and_WasteStream.html>`_
+    `WasteStream <https://qsdsan.readthedocs.io/en/latest/tutorials/WasteStream.html>`_
 
     See Also
     --------
@@ -205,6 +289,10 @@ class WasteStream(SanStream):
                  TMg=None, TCa=None, dry_mass=None, charge=None, ratios=None,
                  stream_impact_item=None, **component_flows):
 
+        if 'impact_item' in component_flows.keys():
+            raise ValueError('The keyword `impact_item` is deprecated, '
+                             'please use `stream_impact_item` instead.')
+
         SanStream.__init__(self=self, ID=ID, flow=flow, phase=phase, T=T, P=P,
                            units=units, price=price, thermo=thermo,
                            stream_impact_item=stream_impact_item, **component_flows)
@@ -236,6 +324,8 @@ class WasteStream(SanStream):
         self._dry_mass = dry_mass
         self._charge = charge
         self._ratios = ratios
+        self._state = None
+        self._dstate = None
 
     @staticmethod
     def from_stream(cls, stream, ID=None, **kwargs):
@@ -434,7 +524,7 @@ class WasteStream(SanStream):
         volatile : bool, optional
             Volatile (True) or involatile (False). The default is None.
         specification : str, optional
-            One of ("SVFA", "XStor", "XANO", "XBio", "SNOx", "XPAO_PP", "TKN").
+            One of ("S_VFA", "X_Stor", "X_ANO", "X_Bio", "S_NOx", "X_PAO_PP", "TKN").
             The default is None.
 
         Returns
@@ -451,7 +541,11 @@ class WasteStream(SanStream):
             raise KeyError(f"Undefined composite variable {variable},"
                            f"Must be one of {_defined_composite_vars}.")
 
-        #!!! assuming it's a liquid WasteStream
+        # Can only be used for liquid
+        if not self.phase == 'l':
+            raise RuntimeError('Only liquid streams can use the `composite` method, '
+                               f'the current WasteStream {self.ID} is {self.phase}.')
+
         #TODO: deal with units
         if subgroup:
             cmps = subgroup
@@ -628,6 +722,24 @@ class WasteStream(SanStream):
     #     return self._liq_sol_properties('charge', self.composite('charge'))
 
 
+
+    @property
+    def iconc(self):
+        '''[Indexer] Mass concentrations, in mg/L (g/m3).'''
+        return self._imol.by_conc(self._thermal_condition)
+
+    @property
+    def conc(self):
+        '''[property_array] Mass concentrations, in mg/L (g/m3).'''
+        return self.iconc.data
+
+
+    @property
+    def Conc(self):
+        '''[property_array] Mass concentrations, in mg/L (g/m3), same as `conc`.'''
+        return self.iconc.data
+
+
     def copy(self, new_ID='', ws_properties=True):
         '''
         Copy the information of another stream.
@@ -639,7 +751,6 @@ class WasteStream(SanStream):
         ws_properties : bool
             Whether to copy wastewater-related properties to the new stream.
 
-
         .. note::
 
             [1] Price of the original stream is not copied.
@@ -658,9 +769,9 @@ class WasteStream(SanStream):
 
     __copy__ = copy
 
-    # TODO: add documents for these functions, differentiate copy, copy_like, and copy_flow
+
     def copy_like(self, other):
-        Stream.copy_like(self, other)
+        SanStream.copy_like(self, other)
 
         if not isinstance(other, WasteStream):
             return
@@ -669,31 +780,6 @@ class WasteStream(SanStream):
             value = getattr(other, slot)
             setattr(self, slot, value)
 
-    def copy_flow(self, other, IDs=..., *, remove=False, exclude=False, if_copy_ws=False):
-        #!!! How to inherit the Stream copy_flow function?
-        # Stream.copy_flow(self, other, IDs, remove, exclude)
-
-        chemicals = self.chemicals
-        mol = other.mol
-        if exclude:
-            IDs = chemicals.get_index(IDs)
-            index = np.ones(chemicals.size, dtype=bool)
-            index[IDs] = False
-        else:
-            index = chemicals.get_index(IDs)
-
-        self.mol[index] = mol[index]
-        if remove:
-            if isinstance(other, MultiStream):
-                other.imol.data[:, index] = 0
-            else:
-                mol[index] = 0
-
-        if if_copy_ws:
-            for slot in _ws_specific_slots:
-                value = getattr(other, slot)
-                setattr(self, slot, value)
-
 
     def mix_from(self, others):
         '''
@@ -731,22 +817,19 @@ class WasteStream(SanStream):
           S_O2         303021.4
           H2O          303021.4
         '''
-        others = [s for s in others if not 'Missing' in type(s).__name__]
-        Stream.mix_from(self, others)
+        SanStream.mix_from(self, others)
 
         for slot in _ws_specific_slots:
             if not hasattr(self, slot) or slot=='_stream_impact_item':
                 continue
-            #!!! This need reviewing, might not be good to calculate some
+            #!!! This needs reviewing, might not be good to calculate some
             # attributes like pH
+            # tot = None
             try:
                 tot = sum(to_float(i, slot)*i.F_vol for i in others if hasattr(i, slot))
-            except: continue
-
-            if tot == 0.:
-                setattr(self, slot, None)
-            else:
                 setattr(self, slot, tot/self.F_vol)
+            except: 
+                setattr(self, slot, None)
 
 
     def get_TDS(self, include_colloidal=True):
@@ -819,7 +902,7 @@ class WasteStream(SanStream):
         return C*conc_unit.conversion_factor(unit)
 
 
-    def set_flow_by_concentration(self, flow_tot, concentrations, units,
+    def set_flow_by_concentration(self, flow_tot, concentrations, units=('L/hr', 'mg/L'),
                                   bulk_liquid_ID='H2O', atol=1e-5, maxiter=50):
         '''
         Set the mass flows of the WasteStream by specifying total volumetric flow and
@@ -848,35 +931,51 @@ class WasteStream(SanStream):
         '''
         if flow_tot == 0: raise RuntimeError(f'{repr(self)} is empty')
         if bulk_liquid_ID in concentrations.keys():
-            C_h2o = concentrations.pop(bulk_liquid_ID)
-            warn(f'ignored concentration specified for {bulk_liquid_ID}:{C_h2o}')
+            C_bulk = concentrations.pop(bulk_liquid_ID)
+            warn(f'ignored concentration specified for {bulk_liquid_ID}:{C_bulk}')
 
         self.empty()
-        f = conc_unit.conversion_factor(units[1])
+        f = conc_unit.conversion_factor(units[1]) # convert to mg/L
         Q_tot = flow_tot / vol_unit.conversion_factor(units[0])   # converted to L/hr
         C_arr = np.array(list(concentrations.values()))
         IDs = concentrations.keys()
-        M_arr = C_arr/f*Q_tot*1e-6       # kg/hr
-        dwt = M_arr.sum()                # dry weight
+        M_arr = C_arr/f*Q_tot*1e-6       # mg/L * L/hr /1e6 = kg/hr
         self.set_flow(M_arr, 'kg/hr', tuple(IDs))
 
+        # Density of the mixture should be larger than the pure bulk liquid,
+        # give it a factor of 100 for safety
         den = self.components[bulk_liquid_ID].rho(phase=self.phase, T=self.T, P=self.P)*1e-3  # bulk liquid density in [kg/L]
-        i = 0
-        while True:
-            i += 1
-            den0 = den
-            M_tot = den0 * Q_tot
-            M_h2o = M_tot - dwt
-            self.set_flow(M_h2o, 'kg/hr', bulk_liquid_ID)
-            den = M_tot / self.get_total_flow('L/hr')
-            if abs(den0 - den) <= atol: break
-            if i > maxiter:
-                raise RuntimeError(f'{bulk_liquid_ID} mass flow calculation failed '
-                                   f'to converge within {maxiter} iterations.')
+        bulk_ref_phase = self.components[bulk_liquid_ID].phase_ref
+        if bulk_ref_phase != 'l':
+            warn(f'Reference phase of liquid is "{bulk_ref_phase}", not "l", '
+                 'flow might not be set correctly.')
 
+        M_bulk_max = Q_tot * den * 100 # L/hr * kg/L = kg/hr
+
+        M_bulk = flx.IQ_interpolation(f=self._Q_obj_f,
+                                      x0=0, x1=M_bulk_max,
+                                      xtol=0.01, ytol=atol, # ytol here is actually Q, but should be fine
+                                      args=(bulk_liquid_ID, Q_tot),
+                                      maxiter=maxiter, checkbounds=False)
+        self.set_flow(M_bulk, 'kg/hr', bulk_liquid_ID)
+
+
+    def _Q_obj_f(self, M_bulk, bulk_liquid_ID, target_Q):
+        self.set_flow(M_bulk, 'kg/hr', bulk_liquid_ID)
+        return self.F_vol*1e3 - target_Q
+
+    def _init_state(self):
+        self._state = np.append(self.conc.astype('float'), self.get_total_flow('m3/d'))
+        self._dstate = self._state * 0.
+    
+    def _state2flows(self):
+        Q = self._state[-1]
+        Cs = dict(zip(self.components.IDs, self._state[:-1]))
+        Cs.pop('H2O', None)
+        self.set_flow_by_concentration(Q, Cs, units=('m3/d', 'mg/L'))
 
     @classmethod
-    def codstates_inf_model(cls, ID, flow_tot=0., units = ('L/hr', 'mg/L'),
+    def codstates_inf_model(cls, ID='', flow_tot=0., units = ('L/hr', 'mg/L'),
                             phase='l', T=298.15, P=101325., price=0., thermo=None,
                             pH=7., SAlk=10., ratios=None,
                             COD=430., TKN=40., TP=10., iVSS_TSS=0.75, iSNH_STKN=0.9,
@@ -893,7 +992,9 @@ class WasteStream(SanStream):
                             X_MgCO3=0., X_CaCO3=0., DO=0., S_H2=0., S_CH4=0.):
 
         if thermo: cmps = thermo.chemicals
-        else: cmps = _load_components(None)
+        else:
+            cmps = _load_components()
+            thermo = _load_thermo()
 
         cmp_dct = dict.fromkeys(cmps.IDs, 0.)
 
@@ -1008,12 +1109,10 @@ class WasteStream(SanStream):
         XB_Substi_N = _calib_XBsub_iN(cmps, cmp_dct, TKN - S_NH4/iSNH_STKN)
         XB_Substi_P = _calib_XBsub_iP(cmps, cmp_dct, TP)
 
-        cmps = Components(cmps)
         if S_NH4 > 0 and cmp_dct['S_F'] > 0: cmps.S_F.i_N = SFi_N
         cmps.X_B_Subst.i_N = XB_Substi_N
         cmps.X_B_Subst.i_P = XB_Substi_P
-        cmps.compile()
-        set_thermo(cmps)
+        cmps.refresh_constants()
 
         #************ convert concentrations to flow rates *************
         new.set_flow_by_concentration(flow_tot, cmp_dct, units)
@@ -1023,7 +1122,7 @@ class WasteStream(SanStream):
 
 
     @classmethod
-    def codbased_inf_model(cls, ID, flow_tot=0., units = ('L/hr', 'mg/L'),
+    def codbased_inf_model(cls, ID='', flow_tot=0., units = ('L/hr', 'mg/L'),
                            phase='l', T=298.15, P=101325., price=0., thermo=None,
                            pH=7., SAlk=10., ratios=None,
                            COD=430., TKN=40., TP=10., iVSS_TSS=0.75, iSNH_STKN=0.9,
@@ -1041,7 +1140,9 @@ class WasteStream(SanStream):
 
 
         if thermo: cmps = thermo.chemicals
-        else: cmps = _load_components(None)
+        else:
+            cmps = _load_components()
+            thermo = _load_thermo()
 
         cmp_dct = dict.fromkeys(cmps.IDs, 0.)
 
@@ -1091,7 +1192,7 @@ class WasteStream(SanStream):
         cmp_c = np.asarray([v for v in cmp_dct.values()])
         S_U = sCOD - (cmp_c * cmps.s * cmps.org).sum()
         cmp_dct['S_U_Inf'] = S_U * r['iSUInf_SU']
-        cmp_dct['S_U_E'] = S_U - cmp_dct['SU_Inf']
+        cmp_dct['S_U_E'] = S_U - cmp_dct['S_U_Inf']
 
         cmp_dct['C_B_BAP'] = C_B * r['iBAP_CB']
         cmp_dct['C_B_UAP'] = C_B * r['iUAP_CB']
@@ -1160,13 +1261,11 @@ class WasteStream(SanStream):
         sub_IDs = ('X_B_Subst', 'X_OHO_PHA', 'X_GAO_PHA', 'X_PAO_PHA', 'X_GAO_Gly', 'X_PAO_Gly')
         fbodtocod_sub = _calib_XBsub_fBODCOD(cmps, cmp_dct, sub_IDs, BOD)
 
-        cmps = Components(cmps)
         if S_NH4 > 0 and cmp_dct['S_F'] > 0: cmps.S_F.i_N = SFi_N
         cmps.X_B_Subst.i_N = XB_Substi_N
         cmps.X_B_Subst.i_P = XB_Substi_P
         for i in sub_IDs: cmps[i].f_BOD5_COD = fbodtocod_sub
-        cmps.compile()
-        set_thermo(cmps)
+        cmps.refresh_constants()
 
         #************ convert concentrations to flow rates *************
         new.set_flow_by_concentration(flow_tot, cmp_dct, units)
@@ -1176,7 +1275,7 @@ class WasteStream(SanStream):
 
 
     @classmethod
-    def bodbased_inf_model(cls, ID, flow_tot=0., units = ('L/hr', 'mg/L'),
+    def bodbased_inf_model(cls, ID='', flow_tot=0., units = ('L/hr', 'mg/L'),
                            phase='l', T=298.15, P=101325., price=0., thermo=None,
                            pH=7., SAlk=10., ratios=None,
                            BOD=250., TKN=40., TP=10., iVSS_TSS=0.75, iSNH_STKN=0.9,
@@ -1194,7 +1293,9 @@ class WasteStream(SanStream):
 
 
         if thermo: cmps = thermo.chemicals
-        else: cmps = _load_components(None)
+        else:
+            cmps = _load_components()
+            thermo = _load_thermo()
 
         cmp_dct = dict.fromkeys(cmps.IDs, 0.)
 
@@ -1245,7 +1346,7 @@ class WasteStream(SanStream):
         cmp_c = np.asarray([v for v in cmp_dct.values()])
         S_U = sCOD - (cmp_c * cmps.s * cmps.org).sum()
         cmp_dct['S_U_Inf'] = S_U * r['iSUInf_SU']
-        cmp_dct['S_U_E'] = S_U - cmp_dct['SU_Inf']
+        cmp_dct['S_U_E'] = S_U - cmp_dct['S_U_Inf']
 
         cmp_dct['C_B_BAP'] = C_B * r['iBAP_CB']
         cmp_dct['C_B_UAP'] = C_B * r['iUAP_CB']
@@ -1314,13 +1415,11 @@ class WasteStream(SanStream):
         sub_IDs = ('X_B_Subst', 'X_OHO_PHA', 'X_GAO_PHA', 'X_PAO_PHA', 'X_GAO_Gly', 'X_PAO_Gly')
         fbodtocod_sub = _calib_XBsub_fBODCOD(cmps, cmp_dct, sub_IDs, BOD)
 
-        cmps = Components(cmps)
         if S_NH4 > 0 and cmp_dct['S_F'] > 0: cmps.S_F.i_N = SFi_N
         cmps.X_B_Subst.i_N = XB_Substi_N
         cmps.X_B_Subst.i_P = XB_Substi_P
         for i in sub_IDs: cmps[i].f_BOD5_COD = fbodtocod_sub
-        cmps.compile()
-        set_thermo(cmps)
+        cmps.refresh_constants()
 
         #************ convert concentrations to flow rates *************
         new.set_flow_by_concentration(flow_tot, cmp_dct, units)
@@ -1330,7 +1429,7 @@ class WasteStream(SanStream):
 
 
     @classmethod
-    def sludge_inf_model(cls, ID, flow_tot=0., units = ('L/hr', 'mg/L'),
+    def sludge_inf_model(cls, ID='', flow_tot=0., units = ('L/hr', 'mg/L'),
                          phase='l', T=298.15, P=101325., price=0., thermo=None,
                          pH=7., SAlk=10., ratios=None,
                          TSS=1e4, TKN=750., TP=250., S_NH4=100., S_PO4=50.,
@@ -1350,7 +1449,9 @@ class WasteStream(SanStream):
 
 
         if thermo: cmps = thermo.chemicals
-        else: cmps = _load_components(None)
+        else:
+            cmps = _load_components()
+            thermo = _load_thermo()
 
         cmp_dct = dict.fromkeys(cmps.IDs, 0.)
 
@@ -1440,7 +1541,7 @@ class WasteStream(SanStream):
         cmp_dct['S_Ac'] = frSAc_scCOD * scCOD
         cmp_dct['S_Prop'] = frSProp_scCOD * scCOD
         cmp_dct['S_U_Inf'] = frSU_scCOD * scCOD * r['iSUInf_SU']
-        cmp_dct['S_U_E'] = frSU_scCOD * scCOD - cmp_dct['SU_Inf']
+        cmp_dct['S_U_E'] = frSU_scCOD * scCOD - cmp_dct['S_U_Inf']
 
         C_B = frCB_scCOD * scCOD
         cmp_dct['C_B_BAP'] = C_B * r['iBAP_CB']
@@ -1464,12 +1565,10 @@ class WasteStream(SanStream):
         XB_Substi_N = _calib_XBsub_iN(cmps, cmp_dct, TKN - S_NH4/iSNH_STKN)
         XB_Substi_P = _calib_XBsub_iP(cmps, cmp_dct, TP)
 
-        cmps = Components(cmps)
         if S_NH4 > 0 and cmp_dct['S_F'] > 0: cmps.S_F.i_N = SFi_N
         cmps.X_B_Subst.i_N = XB_Substi_N
         cmps.X_B_Subst.i_P = XB_Substi_P
-        cmps.compile()
-        set_thermo(cmps)
+        cmps.refresh_constants()
 
         #************ convert concentrations to flow rates *************
         new.set_flow_by_concentration(flow_tot, cmp_dct, units)
