@@ -15,26 +15,27 @@ for license details.
 
 # %%
 
-import numpy as np
+from math import pi, ceil
 from .. import SanUnit, Construction
 from ._decay import Decay
-from ..utils import load_data, data_path
+from ..utils import ospath, load_data, data_path
 
 __all__ = ('AnaerobicDigestion',)
 
-data_path += 'sanunit_data/_anaerobic_digestion.tsv'
+ad_path = ospath.join(data_path, 'sanunit_data/_anaerobic_digestion.tsv')
 
 
 class AnaerobicDigestion(SanUnit, Decay):
     '''
-    Anaerobic digestion of wastes with the production of biogas based on Trimmer et al. [1]_
+    Anaerobic digestion of wastes with the production of biogas based on
+    `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_
 
     Parameters
     ----------
     ins : WasteStream
         Waste for treatment.
     outs : WasteStream
-        Treated waste, biogas, and fugitive N2O.
+        Treated waste, captured biogas, fugitive CH4, and fugitive N2O.
     flow_rate : float
         Total flow rate through the reactor (for sizing purpose), [m3/d].
         If not provided, will use F_vol_in.
@@ -44,7 +45,7 @@ class AnaerobicDigestion(SanUnit, Decay):
         If produced biogas will be captured, otherwise it will be treated
         as fugitive CH4.
     if_N2O_emission : bool
-        If consider N2O emission from N degradation the process.
+        If consider N2O emission from N degradation in the process.
 
     Examples
     --------
@@ -52,10 +53,10 @@ class AnaerobicDigestion(SanUnit, Decay):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     See Also
     --------
@@ -72,7 +73,12 @@ class AnaerobicDigestion(SanUnit, Decay):
         self.if_capture_biogas = if_capture_biogas
         self.if_N2O_emission = if_N2O_emission
 
-        data = load_data(path=data_path)
+        self.construction = (
+            Construction('concrete', item='Concrete', quantity_unit='m3'),
+            Construction('excavation', item='Excavation', quantity_unit='m3'),
+            )
+
+        data = load_data(path=ad_path)
         for para in data.index:
             value = float(data.loc[para]['expected'])
             setattr(self, '_'+para, value)
@@ -93,7 +99,8 @@ class AnaerobicDigestion(SanUnit, Decay):
         biogas.phase = CH4.phase = N2O.phase = 'g'
 
         # COD removal
-        COD_deg = treated._COD*treated.F_vol/1e3*self.COD_removal # kg/hr
+        _COD = waste._COD or waste.COD
+        COD_deg = _COD*treated.F_vol/1e3*self.COD_removal # kg/hr
         treated._COD *= (1-self.COD_removal)
         treated.imass[self.degraded_components] *= (1-self.COD_removal)
 
@@ -137,13 +144,14 @@ class AnaerobicDigestion(SanUnit, Decay):
         design['Single reactor volume'] = V_single = V_tot/(1-self.headspace_frac)/(N-1)
 
         # Rx modeled as a cylinder
-        design['Reactor diameter'] = D = (4*V_single*self.aspect_ratio/np.pi)**(1/3)
+        design['Reactor diameter'] = D = (4*V_single*self.aspect_ratio/pi)**(1/3)
         design['Reactor height'] = H = self.aspect_ratio * D
-        concrete =  N*self.concrete_thickness*(2*np.pi/4*(D**2)+np.pi*D*H)
-        self.construction = (
-            Construction(item='Concrete', quantity=concrete, quantity_unit='m3'),
-            Construction(item='Excavation', quantity=V_tot, quantity_unit='m3'),
-            )
+
+        constr = self.construction
+        concrete =  N*self.concrete_thickness*(2*pi/4*(D**2)+pi*D*H)
+        constr[0].quantity = concrete
+        constr[1].quantity = V_tot # excavation
+
         self.add_construction()
 
 
@@ -180,7 +188,7 @@ class AnaerobicDigestion(SanUnit, Decay):
         return self._N_reactor
     @N_reactor.setter
     def N_reactor(self, i):
-        self._N_reactor = int(np.ceil(i))
+        self._N_reactor = ceil(i)
 
     @property
     def aspect_ratio(self):
