@@ -16,19 +16,20 @@ for license details.
 # %%
 
 import numpy as np
+from math import ceil
 from warnings import warn
 from .. import SanUnit, Construction
 from ._decay import Decay
-from ..utils import load_data, data_path, dct_from_str
+from ..utils import ospath, load_data, data_path, dct_from_str
 
 __all__ = ('DryingBed',)
 
-data_path += 'sanunit_data/_drying_bed.tsv'
-
+drying_bed_path = ospath.join(data_path, 'sanunit_data/_drying_bed.tsv')
 
 class DryingBed(SanUnit, Decay):
     '''
-    Unplanted and planted drying bed for solids based on Trimmer et al. [1]_
+    Unplanted and planted drying bed for solids based on
+    `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_
 
     Parameters
     ----------
@@ -49,10 +50,10 @@ class DryingBed(SanUnit, Decay):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     See Also
     --------
@@ -77,7 +78,12 @@ class DryingBed(SanUnit, Decay):
 
         self.degraded_components = tuple(degraded_components)
 
-        data = load_data(path=data_path)
+        self.construction = (
+            Construction('concrete', item='Concrete', quantity_unit='m3'),
+            Construction('steel', item='Steel', quantity_unit='kg'),
+            )
+
+        data = load_data(path=drying_bed_path)
         for para in data.index:
             if para == 'N_bed': continue
             if para in ('sol_frac', 'bed_L', 'bed_W', 'bed_H'):
@@ -103,7 +109,8 @@ class DryingBed(SanUnit, Decay):
         COD_loss = self.first_order_decay(k=self.decay_k_COD,
                                           t=self.tau/365,
                                           max_decay=self.COD_max_decay)
-        sol_COD = sol._COD/1e3*sol.F_vol * (1-COD_loss)
+        _COD = sol._COD or sol.COD
+        sol_COD = _COD/1e3*sol.F_vol * (1-COD_loss)
         sol.imass[self.degraded_components] *= 1 - COD_loss
         CH4.imass['CH4'] = waste.COD/1e3*waste.F_vol*COD_loss * \
             self.max_CH4_emission*self.MCF_decay # COD in mg/L (g/m3)
@@ -163,10 +170,11 @@ class DryingBed(SanUnit, Decay):
         concrete = (N*self.concrete_thickness*(L*W+2*L*H+2*W*H)).sum()
         steel = tot_cover_area*self.cover_unit_mass + \
             tot_column_length*self.column_unit_mass
-        self.construction = (
-            Construction(item='Concrete', quantity=concrete, quantity_unit='m3'),
-            Construction(item='Steel', quantity=steel, quantity_unit='kg'),
-            )
+
+        constr = self.construction
+        constr[0].quantity = concrete
+        constr[1].quantity = steel
+
         for i in self.construction:
             self.F_BM[i.item.ID] = 1
 
@@ -206,11 +214,11 @@ class DryingBed(SanUnit, Decay):
         float will be converted to the smallest integer.
         '''
         for i, j in self._N_bed.items():
-            self._N_bed[i] = int(np.ceil(j))
+            self._N_bed[i] = ceil(j)
         return self._N_bed
     @N_bed.setter
     def N_bed(self, i):
-        int_i = {k: np.ceil(v) for k, v in i.items()}
+        int_i = {k: ceil(v) for k, v in i.items()}
         self._N_bed.update(int_i)
 
     @property
@@ -251,7 +259,7 @@ class DryingBed(SanUnit, Decay):
         return self._column_per_side
     @column_per_side.setter
     def column_per_side(self, i):
-        self._column_per_side = int(np.ceil(i))
+        self._column_per_side = ceil(i)
 
     @property
     def column_unit_mass(self):
