@@ -15,18 +15,21 @@ for license details.
 
 # %%
 
-import numpy as np
 from warnings import warn
+from math import ceil
 from .. import SanUnit, Construction
 from ._decay import Decay
-from ..utils import load_data, data_path
+from ..utils import ospath, load_data, data_path
 
 __all__ = ('Lagoon',)
 
+anaerobic_path = ospath.join(data_path, 'sanunit_data/_anaerobic_lagoon.tsv')
+facultative_path = ospath.join(data_path, 'sanunit_data/_facultative_lagoon.tsv')
 
 class Lagoon(SanUnit, Decay):
     '''
-    Anaerobic and facultative lagoon treatment based on Trimmer et al. [1]_
+    Anaerobic and facultative lagoon treatment based on
+    `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_
 
     Parameters
     ----------
@@ -42,7 +45,7 @@ class Lagoon(SanUnit, Decay):
     degraded_components : tuple
         IDs of components that will degrade (at the same removal as `COD_removal`).
     if_N2O_emission : bool
-        If consider N2O emission from N degradation the process.
+        If consider N2O emission from N degradation in the process.
 
     Examples
     --------
@@ -50,10 +53,10 @@ class Lagoon(SanUnit, Decay):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     See Also
     --------
@@ -67,18 +70,19 @@ class Lagoon(SanUnit, Decay):
         SanUnit.__init__(self, ID, ins, outs, thermo, init_with)
         self._tau = None
         self._P_removal = 0.
-
-        anaerobic_path = data_path + 'sanunit_data/_anaerobic_lagoon.tsv'
         self._anaerobic_defaults = load_data(path=anaerobic_path)
-        facultative_path = data_path + 'sanunit_data/_facultative_lagoon.tsv'
         self._facultative_defaults = load_data(path=facultative_path)
-
         self._design_type = None
         self.design_type = design_type
         self._flow_rate = flow_rate
         self.degraded_components = tuple(degraded_components)
         self.if_N2O_emission = if_N2O_emission
 
+        self.construction = (
+            Construction('liner', item='Plastic', quantity_unit='kg'),
+            Construction('excavation', item='Excavation', quantity_unit='m3'),
+            )
+
         for attr, value in kwargs.items():
             setattr(self, attr, value)
 
@@ -94,7 +98,8 @@ class Lagoon(SanUnit, Decay):
         removed_frac = self.COD_removal*self.COD_decay
         treated.imass[self.degraded_components] *= 1 - self.COD_removal
 
-        CH4.imass['CH4'] = removed_frac*waste.COD*waste.F_vol/1e3 * \
+        _COD = waste._COD or waste.COD
+        CH4.imass['CH4'] = removed_frac*_COD*waste.F_vol/1e3 * \
             self.MCF_decay*self.max_CH4_emission
 
         if self.if_N2O_emission:
@@ -111,7 +116,7 @@ class Lagoon(SanUnit, Decay):
             N2O.empty()
 
         treated.imass['P'] *= 1 - self.P_removal
-        treated._COD = waste.COD*waste.F_vol*(1-self.COD_removal)/treated.F_vol
+        treated._COD = _COD*waste.F_vol*(1-self.COD_removal)/treated.F_vol
 
     _units = {
         'Single lagoon volume': 'm3',
@@ -129,10 +134,10 @@ class Lagoon(SanUnit, Decay):
         design['Lagoon depth'] = depth = V / (L*W)
 
         liner = (L*W + 2*depth*(L+W)) * N * self.liner_unit_mass
-        self.construction = (
-            Construction(item='Plastic', quantity=liner, quantity_unit='kg'),
-            Construction(item='Excavation', quantity=N*V, quantity_unit='m3'),
-            )
+        constr = self.construction
+        constr[0].quantity = liner
+        constr[1].quantity = N * V # excavation
+
         self.add_construction(add_cost=False)
 
     @property
@@ -187,7 +192,7 @@ class Lagoon(SanUnit, Decay):
         return self._N_lagoon
     @N_lagoon.setter
     def N_lagoon(self, i):
-        self._N_lagoon = int(np.ceil(i))
+        self._N_lagoon = ceil(i)
 
     @property
     def flow_rate(self):
