@@ -15,19 +15,20 @@ for license details.
 
 # %%
 
-import numpy as np
+from math import ceil
 from .. import SanUnit, Construction
 from ._decay import Decay
-from ..utils import load_data, data_path
+from ..utils import ospath, load_data, data_path
 
 __all__ = ('LiquidTreatmentBed',)
 
-data_path += 'sanunit_data/_liquid_treatment_bed.tsv'
+liquid_bed_path = ospath.join(data_path, 'sanunit_data/_liquid_treatment_bed.tsv')
 
 
 class LiquidTreatmentBed(SanUnit, Decay):
     '''
-    For secondary treatment of liquid based on Trimmer et al. [1]_
+    For secondary treatment of liquid based on
+    `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_
 
     Parameters
     ----------
@@ -38,7 +39,7 @@ class LiquidTreatmentBed(SanUnit, Decay):
     degraded_components : tuple
         IDs of components that will degrade (at the same removal as `COD_removal`).
     if_N2O_emission : bool
-        If consider N2O emission from N degradation the process.
+        If consider N2O emission from N degradation in the process.
 
     Examples
     --------
@@ -46,10 +47,10 @@ class LiquidTreatmentBed(SanUnit, Decay):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     See Also
     --------
@@ -62,8 +63,9 @@ class LiquidTreatmentBed(SanUnit, Decay):
         SanUnit.__init__(self, ID, ins, outs, thermo, init_with, F_BM_default=1)
         self.degraded_components = tuple(degraded_components)
         self.if_N2O_emission = if_N2O_emission
+        self.construction = (Construction('concrete', item='Concrete', quantity_unit='m3'))
 
-        data = load_data(path=data_path)
+        data = load_data(path=liquid_bed_path)
         for para in data.index:
             value = float(data.loc[para]['expected'])
             setattr(self, '_'+para, value)
@@ -82,12 +84,13 @@ class LiquidTreatmentBed(SanUnit, Decay):
         CH4.phase = N2O.phase = 'g'
 
         # COD removal
+        _COD = waste._COD or waste.COD
         COD_loss = self.first_order_decay(k=self.decay_k_COD,
                                           t=self.tau/365,
                                           max_decay=self.COD_max_decay)
-        COD_loss_tot = COD_loss*waste.COD/1e3*waste.F_vol
+        COD_loss_tot = COD_loss*_COD/1e3*waste.F_vol
 
-        treated._COD *= (1-COD_loss)
+        treated._COD = _COD * (1-COD_loss)
         treated.imass[self.degraded_components] *= (1-COD_loss)
 
         CH4_prcd = COD_loss_tot*self.MCF_decay*self.max_CH4_emission
@@ -124,9 +127,7 @@ class LiquidTreatmentBed(SanUnit, Decay):
         design['Single bed volume'] = L*W*H
 
         concrete = N*self.concrete_thickness*(L*W+2*L*H+2*W*H)
-        self.construction = (
-            Construction(item='Concrete', quantity=concrete, quantity_unit='m3'),
-            )
+        self.construction[0].quantity = concrete
         self.add_construction()
 
 
@@ -148,7 +149,7 @@ class LiquidTreatmentBed(SanUnit, Decay):
         return self._N_bed
     @N_bed.setter
     def N_bed(self, i):
-        self._N_bed = int(np.ceil(i))
+        self._N_bed = ceil(i)
 
     @property
     def bed_L(self):
