@@ -17,18 +17,17 @@ for license details.
 
 from .. import WasteStream, Construction
 from ._toilet import Toilet
-from ..utils import load_data, data_path, dct_from_str
+from ..utils import ospath, load_data, data_path, dct_from_str
 
 __all__ = ('PitLatrine',)
 
-data_path += 'sanunit_data/_pit_latrine.tsv'
+pit_path = ospath.join(data_path, 'sanunit_data/_pit_latrine.tsv')
 
 
-# %%
-
 class PitLatrine(Toilet):
     '''
-    Single pit latrine based on Trimmer et al. [1]_, a subclass of :class:`~.Toilet`.
+    Single pit latrine based on `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_,
+    a subclass of :class:`~.Toilet`.
 
     Parameters
     ----------
@@ -52,14 +51,14 @@ class PitLatrine(Toilet):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     See Also
     --------
-    :ref:`qsdsan.sanunits.Toilet <sanunits_Toilet>`
+    :ref:`qsdsan.sanunits.Toilet <sanunits_toilets>`
     '''
 
     # Legacy code to add checkers
@@ -82,8 +81,22 @@ class PitLatrine(Toilet):
         self.if_leaching = if_leaching
         self.if_pit_above_water_table = if_pit_above_water_table
         self.if_shared = if_shared
+        self._pit_depth = 4.57 # m
+        self._pit_area = 0.8 # m2
+        self._liq_leaching = None
 
-        data = load_data(path=data_path)
+        self.construction = (
+            Construction('cement', item='Cement', quantity_unit='kg'),
+            Construction('sand', item='Sand', quantity_unit='kg'),
+            Construction('gravel', item='Gravel', quantity_unit='kg'),
+            Construction('brick', item='Brick', quantity_unit='kg'),
+            Construction('liner', item='Plastic', quantity_unit='kg'),
+            Construction('steel', item='Steel', quantity_unit='kg'),
+            Construction('wood', item='Wood', quantity_unit='m3'),
+            Construction('excavation', item='Excavation', quantity_unit='m3'),
+            )
+
+        data = load_data(path=pit_path)
         for para in data.index:
             if para in ('MCF_decay', 'N2O_EF_decay'):
                 value = dct_from_str(data.loc[para]['expected'], dtype='float')
@@ -92,9 +105,6 @@ class PitLatrine(Toilet):
             setattr(self, '_'+para, value)
         del data
 
-        self._pit_depth = 4.57 # m
-        self._pit_area = 0.8 # m2
-        self._liq_leaching = None
         for attr, value in kwargs.items():
             setattr(self, attr, value)
 
@@ -108,11 +118,11 @@ class PitLatrine(Toilet):
 
         mixed = WasteStream()
         mixed.mix_from(self.ins)
-        tot_COD_kg = sum(float(getattr(i, 'COD'))*i.F_vol for i in self.ins)/1e3
+        tot_COD_kg = sum(float(getattr(i, '_COD') or getattr(i, 'COD'))*i.F_vol for i in self.ins)/1e3
 
         # All composite variables in mg/L
         # Leaching
-        # Here COD change due to leaching not considered
+        # Here COD change due to leaching is not considered
         if self.if_leaching:
             # Additional assumption not in ref [1]
             leachate.imass['H2O'] = mixed.imass['H2O'] * self.liq_leaching
@@ -124,7 +134,6 @@ class PitLatrine(Toilet):
             mixed.mass -= leachate.mass
 
         # Air emission
-        #!!! Based on the logic, COD won't degrade without air emission?
         if self.if_air_emission:
             # N loss due to ammonia volatilization
             NH3_rmd, NonNH3_rmd = \
@@ -163,7 +172,7 @@ class PitLatrine(Toilet):
                 CH4_factor=self.COD_max_decay*self.MCF_aq*self.max_CH4_emission,
                 N2O_factor=self.N2O_EF_decay*44/28)
 
-        # Drain extra water, assume density of water (1 kg/L)
+        # Drain extra water, assume density of water to be 1 kg/L
         sludge = self.sludge_accum_rate/(365*24)
         diff = mixed.F_mass - sludge
         if diff > 0:
@@ -198,16 +207,15 @@ class PitLatrine(Toilet):
         design['Single pit depth'] = self.pit_depth
 
         density = self.density_dct
-        self.construction = (
-            Construction(item='Cement', quantity=700*N, quantity_unit='kg'),
-            Construction(item='Sand', quantity=2.2*density['Sand']*N, quantity_unit='kg'),
-            Construction(item='Gravel', quantity=0.8*density['Gravel']*N, quantity_unit='kg'),
-            Construction(item='Brick', quantity=54*0.0024*density['Brick']*N, quantity_unit='kg'),
-            Construction(item='Plastic', quantity=16*density['Plastic']*N, quantity_unit='kg'),
-            Construction(item='Steel', quantity=0.00425*density['Steel']*N, quantity_unit='kg'),
-            Construction(item='Wood', quantity=0.19*N, quantity_unit='m3'),
-            Construction(item='Excavation', quantity=self.pit_V*N, quantity_unit='m3'),
-            )
+        constr = self.construction
+        constr[0].quantity = 700 * N # cment
+        constr[1].quantity = 2.2 * density['Sand'] * N
+        constr[2].quantity = 0.8 * density['Gravel'] * N
+        constr[3].quantity = 54*0.0024 * density['Brick'] * N
+        constr[4].quantity = 16 * density['Plastic'] * N
+        constr[5].quantity = 0.00425 * density['Steel'] * N
+        constr[6].quantity = 0.19 * N # wood
+        constr[7].quantity = self.pit_V * N # excavation
 
         self.add_construction(add_cost=False)
 
