@@ -15,20 +15,21 @@ for license details.
 
 # %%
 
-import numpy as np
+from math import ceil, pi, cos
 from .. import Construction
 from ._decay import Decay
 from ._sludge_separator import SludgeSeparator
-from ..utils import load_data, data_path
+from ..utils import ospath, load_data, data_path
 
 __all__ = ('SedimentationTank',)
 
-data_path += 'sanunit_data/_sedimentation_tank.tsv'
+sedmentation_path = ospath.join(data_path, 'sanunit_data/_sedimentation_tank.tsv')
 
 
 class SedimentationTank(SludgeSeparator, Decay):
     '''
-    Sedimentation of wastes into liquid and solid phases based on Trimmer et al. [1]_
+    Sedimentation of wastes into liquid and solid phases based on
+    `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_
 
     Parameters
     ----------
@@ -39,7 +40,7 @@ class SedimentationTank(SludgeSeparator, Decay):
     degraded_components : tuple
         IDs of components that will degrade (simulated by first-order decay).
     if_N2O_emission : bool
-        If consider N2O emission from N degradation the process.
+        If consider N2O emission from N degradation in the process.
 
     Examples
     --------
@@ -47,10 +48,10 @@ class SedimentationTank(SludgeSeparator, Decay):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     See Also
     --------
@@ -66,7 +67,12 @@ class SedimentationTank(SludgeSeparator, Decay):
         self.degraded_components = tuple(degraded_components)
         self.if_N2O_emission = if_N2O_emission
 
-        data = load_data(path=data_path)
+        self.construction = (
+            Construction('concrete', item='Concrete', quantity_unit='m3'),
+            Construction('steel', item='Steel', quantity_unit='kg'),
+            )
+
+        data = load_data(path=sedmentation_path)
         for para in data.index:
             value = float(data.loc[para]['expected'])
             setattr(self, '_'+para, value)
@@ -90,7 +96,9 @@ class SedimentationTank(SludgeSeparator, Decay):
         COD_loss = self.first_order_decay(k=self.decay_k_COD,
                                           t=self.tau/365,
                                           max_decay=self.COD_max_decay)
-        tot_COD_kg = sol._COD * sol.F_vol / 1e3
+
+        _COD = sol._COD or sol.COD
+        tot_COD_kg = _COD * sol.F_vol / 1e3
         sol.imass[self.degraded_components] *= 1 - COD_loss
 
         # Adjust total mass of of the settled solids by changing water content
@@ -124,7 +132,7 @@ class SedimentationTank(SludgeSeparator, Decay):
 
     def _design(self):
         design = self.design_results
-        #!!! Why isn't tau used?
+        # `tau` not used, might be that working volume fraction not known
         design['Tank number'] = N = self.N_tank
         design['Single tank volume'] = V_single = self.tank_V
         L2W = self.tank_L_to_W
@@ -132,7 +140,7 @@ class SedimentationTank(SludgeSeparator, Decay):
         design['Single tank height'] = H = (V_single/(L2W*(W2H**2)))**(1/3)
         design['Single tank width'] = W = H * W2H
         design['Single tank length'] = L = W * L2W
-        design['Single roof area'] = N*L*W/(np.cos(self.roof_slope/180*np.pi))
+        design['Single roof area'] = N*L*W/(cos(self.roof_slope/180*pi))
         side_area = N*2*(L*H + W*H)
 
         # Concrete
@@ -140,10 +148,10 @@ class SedimentationTank(SludgeSeparator, Decay):
         side_concrete = N*thick*(L*W+2*W*H+2*L*H)
         column_concrete = N*(thick**2)*H*self.column_per_side*2
 
-        self.construction = (
-            Construction(item='Concrete', quantity=side_concrete+column_concrete, quantity_unit='m3'),
-            Construction(item='Excavation', quantity=design['Single roof area']+side_area, quantity_unit='m3'),
-            )
+        constr = self.construction
+        constr[0].quantity = side_concrete + column_concrete
+        constr[1].quantity = (design['Single roof area']+side_area) * self.roof_unit_mass # steel
+
         self.add_construction()
 
     @property
@@ -184,7 +192,7 @@ class SedimentationTank(SludgeSeparator, Decay):
         return self._N_tank
     @N_tank.setter
     def N_tank(self, i):
-        self._N_tank = int(np.ceil(i))
+        self._N_tank = ceil(i)
 
     @property
     def column_per_side(self):
@@ -192,7 +200,7 @@ class SedimentationTank(SludgeSeparator, Decay):
         return self._column_per_side
     @column_per_side.setter
     def column_per_side(self, i):
-        self._column_per_side = int(np.ceil(i))
+        self._column_per_side = ceil(i)
 
     @property
     def concrete_thickness(self):
