@@ -18,18 +18,19 @@ for license details.
 from warnings import warn
 from .. import SanUnit
 from ._decay import Decay
-from ..utils import load_data, data_path, dct_from_str
+from ..utils import ospath, load_data, data_path, dct_from_str
 
 __all__ = ('SludgeSeparator',)
 
-data_path += 'sanunit_data/_sludge_separator.tsv'
+separator_path = ospath.join(data_path, 'sanunit_data/_sludge_separator.tsv')
 
 allocate_N_removal = Decay.allocate_N_removal
 
 class SludgeSeparator(SanUnit):
     '''
-    For sludge separation based on Trimmer et al. [1]_, note that no default
-    cost or environmental impacts are included.
+    For sludge separation based on
+    `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_,
+    note that no default cost or environmental impacts are included.
 
     Parameters
     ----------
@@ -50,10 +51,10 @@ class SludgeSeparator(SanUnit):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     '''
 
@@ -61,15 +62,12 @@ class SludgeSeparator(SanUnit):
                  split=None, settled_frac=None, **kwargs):
         SanUnit.__init__(self, ID, ins, outs, thermo, init_with, **kwargs)
 
-        data = load_data(path=data_path)
-        if not split:
-            value = dct_from_str(data.loc['split']['expected'])
-            setattr(self, 'split', value)
-        if not settled_frac:
-            value = float(data.loc['settled_frac']['expected'])
-            setattr(self, 'settled_frac', value)
+        data = load_data(path=separator_path)
+        self.split = split or dct_from_str(data.loc['split']['expected'])
+        self.settled_frac = settled_frac or float(data.loc['settled_frac']['expected'])
         del data
 
+
     _N_ins = 1
     _outs_size_is_fixed = False
 
@@ -78,7 +76,7 @@ class SludgeSeparator(SanUnit):
         sol.imass['H2O'] = influent.F_mass * self.settled_frac - sol.F_mass
         if sol.imass['H2O'] < 0:
             sol.imass['H2O'] = 0
-            msg = 'Negative water content calcualted for settled solids, ' \
+            msg = 'Negative water content calculated for settled solids, ' \
                 'try smaller split or larger settled_frac.'
             warn(msg, stacklevel=2)
         liq.imass['H2O'] = influent.imass['H2O'] - sol.imass['H2O']
@@ -89,6 +87,7 @@ class SludgeSeparator(SanUnit):
         liq, sol = self.outs[0], self.outs[1]
 
         # Retention in the settled solids
+        sol_COD = liq_COD = None
         split = self.split
         if self._split_type == 'float':
             liq.copy_like(waste)
@@ -97,13 +96,12 @@ class SludgeSeparator(SanUnit):
             liq.mass -= sol.mass
         else:
             for var in self.split.keys():
-                #!!! In the future this should be best done by changing the state variable
                 if var == 'TS':
-                    try: sol.imass['OtherSS'] = split[var] * waste.imass['OtherSS']
-                    except: breakpoint()
+                    sol.imass['OtherSS'] = split[var] * waste.imass['OtherSS']
                 elif var == 'COD':
-                    sol_COD = split[var] * waste._COD * waste.F_vol
-                    liq_COD = waste._COD * waste.F_vol - sol_COD
+                    _COD = waste._COD or waste.COD
+                    sol_COD = split[var] * _COD * waste.F_vol
+                    liq_COD = _COD * waste.F_vol - sol_COD
                 elif var == 'N':
                     N_sol = split[var]*(waste.imass['NH3']+waste.imass['NonNH3'])
                     NonNH3_rmd, NH3_rmd = \
@@ -116,8 +114,8 @@ class SludgeSeparator(SanUnit):
 
         # Adjust total mass of of the settled solids by changing water content.
         liq, sol = self._adjust_solid_water(waste, liq, sol)
-        sol._COD = sol_COD / sol.F_vol
-        liq._COD = liq_COD / liq.F_vol
+        sol._COD = sol._COD if not sol_COD else sol_COD / sol.F_vol
+        liq._COD = liq._COD if not liq_COD else liq_COD / liq.F_vol
 
 
     @property
