@@ -18,11 +18,11 @@ for license details.
 import numpy as np
 from .. import Construction
 from ._toilet import Toilet
-from ..utils import load_data, data_path
+from ..utils import ospath, load_data, data_path
 
 __all__ = ('UDDT',)
 
-data_path += 'sanunit_data/_uddt.tsv'
+uddt_path = ospath.join(data_path, 'sanunit_data/_uddt.tsv')
 
 
 # %%
@@ -30,7 +30,8 @@ data_path += 'sanunit_data/_uddt.tsv'
 class UDDT(Toilet):
     '''
     Urine-diverting dry toilet with liquid storage tank and dehydration vault
-    for urine and feces storage, respectively, based on Trimmer et al. [1]_,
+    for urine and feces storage, respectively, based on
+    `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_,
     a subclass of qsdsan.sanunits.Toilet.
 
     Parameters
@@ -49,8 +50,6 @@ class UDDT(Toilet):
         must be larger than 1.
     if_treatment : bool
         If has onsite treatment.
-    if_pit_above_water_table : bool
-        If the pit is above local water table.
 
     Examples
     --------
@@ -58,19 +57,19 @@ class UDDT(Toilet):
 
     References
     ----------
-    .. [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
-        Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
-        Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
-        https://doi.org/10.1021/acs.est.0c03296.
+    [1] Trimmer et al., Navigating Multidimensional Social–Ecological System
+    Trade-Offs across Sanitation Alternatives in an Urban Informal Settlement.
+    Environ. Sci. Technol. 2020, 54 (19), 12641–12653.
+    https://doi.org/10.1021/acs.est.0c03296.
 
     See Also
     --------
-    :ref:`qsdsan.sanunits.Toilet <sanunits_Toilet>`
+    :ref:`qsdsan.sanunits.Toilet <sanunits_toilets>`
     '''
 
     def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
                  degraded_components=('OtherSS',), N_user=1, N_toilet=1, lifetime=8,
-                 if_toilet_paper=True, if_flushing=True, if_cleansing=False,
+                 if_toilet_paper=True, if_flushing=False, if_cleansing=False,
                  if_desiccant=True, if_air_emission=True, if_ideal_emptying=True,
                  CAPEX=553, OPEX_over_CAPEX=0.1,
                  T=273.15+24, safety_factor=1, if_prep_loss=True, if_treatment=False,
@@ -86,7 +85,18 @@ class UDDT(Toilet):
         self.if_prep_loss = if_prep_loss
         self.if_treatment = if_treatment
 
-        data = load_data(path=data_path)
+        self.construction = (
+            Construction('cement', item='Cement', quantity_unit='kg'),
+            Construction('sand', item='Sand', quantity_unit='kg'),
+            Construction('gravel', item='Gravel', quantity_unit='kg'),
+            Construction('brick', item='Brick', quantity_unit='kg'),
+            Construction('liner', item='Plastic', quantity_unit='kg'),
+            Construction('steel', item='Steel', quantity_unit='kg'),
+            Construction('ss_sheet', item='StainlessSteelSheet', quantity_unit='kg'),
+            Construction('wood', item='Wood', quantity_unit='m3'),
+            )
+
+        data = load_data(path=uddt_path)
         for para in data.index:
             value = float(data.loc[para]['expected'])
             setattr(self, '_'+para, value)
@@ -104,13 +114,15 @@ class UDDT(Toilet):
         liq.copy_like(self.ins[0])
         # Assume all of additives (toilet paper and desiccant) and water
         # (cleansing and flushing) go to feces
-        sol_COD = self.ins[1]._COD*self.ins[1].F_vol/1e3
+        _COD = self.ins[1]._COD or self.ins[1].COD
+        sol_COD = _COD*self.ins[1].F_vol/1e3
         sol.mix_from(self.ins[1:])
         sol._COD = sol_COD*1e3/sol.F_vol
         struvite.phase = HAP.phase = 's'
         CH4.phase = N2O.phase = 'g'
 
-        #!!! Modified from ref [1], assume this only happens when air emission occurs
+        # Modified from ref [1], assume this only happens when air emission occurs
+        # (to be consistent with pit latrine)
         if self.if_air_emission:
             # N loss due to ammonia volatilization
             NH3_rmd, NonNH3_rmd = \
@@ -122,7 +134,7 @@ class UDDT(Toilet):
             COD_loss = self.first_order_decay(k=self.decay_k_COD,
                                               t=self.collection_period/365,
                                               max_decay=self.COD_max_decay)
-            CH4.imass['CH4'] = sol.COD/1e3*sol.F_vol*COD_loss * \
+            CH4.imass['CH4'] = sol._COD/1e3*sol.F_vol*COD_loss * \
                 self.max_CH4_emission*self.MCF_decay # COD in mg/L (g/m3)
             sol._COD *= 1 - COD_loss
             sol.imass[self.degraded_components] *= 1 - COD_loss
@@ -187,7 +199,6 @@ class UDDT(Toilet):
                 (alpha + ((1-alpha)*(f_NH3_Emerson**beta)))
             NH3_conc = NH3_mmol * f_NH3_Pitzer
 
-            #!!! Shouldn't the collection period and design be affected by this as well?
             # Time (in days) to reach desired inactivation level
             self.treatment_tau = ((3.2 + self.log_removal) \
                              / (10**(-3.7+0.062*(self.T-273.15)) * (NH3_conc**0.7))) \
@@ -207,7 +218,9 @@ class UDDT(Toilet):
             mixed.mix_from(self.ins[1:])
             fec_moi_int = mixed.imass['H2O']/mixed.F_mass
             fec_moi = MC_min + (fec_moi_int-MC_min)/(r*t)*(1-np.exp(-r*t))
-            sol.imass['H2O'] = (sol.F_mass-sol.imass['H2O']) / (1-fec_moi)
+            dry_mass = sol.F_mass-sol.imass['H2O']
+            tot_mass = (sol.F_mass-sol.imass['H2O']) / (1-fec_moi)
+            sol.imass['H2O'] = tot_mass - dry_mass
             sol._COD = sol_COD*1e3/sol.F_vol
 
         self._vault_V = sol.F_vol*self.collection_period*24 # in day
@@ -228,10 +241,12 @@ class UDDT(Toilet):
         # Scale up the effluent based on the number of user per toilet and
         # toilet number
         tot_user = self.N_user * self.N_toilet
+
         for i in self.outs:
             if not i.F_mass == 0:
                 i.F_mass *= tot_user
 
+
     _units = {
         'Collection period': 'd',
         'Single tank volume': 'm3',
@@ -251,17 +266,15 @@ class UDDT(Toilet):
         design['Treatment volume'] = self.treatment_V
 
         density = self.density_dct
-        self.construction = (
-            Construction(item='Cement', quantity=200*N, quantity_unit='kg'),
-            Construction(item='Sand', quantity=0.6*density['Sand']*N, quantity_unit='kg'),
-            Construction(item='Gravel', quantity=0.2*density['Gravel']*N, quantity_unit='kg'),
-            Construction(item='Brick', quantity=682*0.0024*density['Brick']*N, quantity_unit='kg'),
-            Construction(item='Plastic', quantity=4*density['Plastic']*N, quantity_unit='kg'),
-            Construction(item='Steel', quantity=0.00351*density['Steel']*N, quantity_unit='kg'),
-            Construction(item='StainlessSteelSheet',
-                         quantity=28.05*density['StainlessSteelSheet']*N, quantity_unit='kg'),
-            Construction(item='Wood', quantity=0.222*N, quantity_unit='m3'),
-            )
+        constr = self.construction
+        constr[0].quantity = 200 * N # cement
+        constr[1].quantity = 0.6 * density['Sand'] * N
+        constr[2].quantity = 0.2 * density['Gravel'] * N
+        constr[3].quantity = 682*0.0024 * density['Brick'] * N
+        constr[4].quantity = 4 * density['Plastic'] * N
+        constr[5].quantity = 0.00351 * density['Steel'] * N
+        constr[6].quantity = 28.05*density['StainlessSteelSheet'] * N
+        constr[7].quantity = 0.222 * N # wood
 
         self.add_construction(add_cost=False)
 
@@ -323,7 +336,8 @@ class UDDT(Toilet):
     @property
     def prep_sludge(self):
         '''
-        [float] Fraction of total precipitate appearing as sludge that can
+        [float]
+        Fraction of total precipitate appearing as sludge that can
         settle and be removed.
         '''
         return self._prep_sludge
