@@ -8,8 +8,8 @@ This module is developed by:
     Yalin Li <zoe.yalin.li@gmail.com>
 
 With contributions from:
-    Joy Cheung <joycheung1994@gmail.com>
-    Yoel Rene Cortés-Peña <yoelcortes@gmail.com>
+    Joy Zhang <joycheung1994@gmail.com>
+    Yoel Cortés-Peña <yoelcortes@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
@@ -20,9 +20,11 @@ for license details.
 # %%
 
 __all__ = ('get_correlations', 'define_inputs', 'generate_samples',
-           'morris_analysis', 'morris_till_convergence', 'sobol_analysis',
+           'morris_analysis', 'morris_till_convergence',
+           'fast_analysis', 'sobol_analysis',
            'plot_uncertainties', 'plot_correlations',
-           'plot_morris_results', 'plot_morris_convergence', 'plot_sobol_results')
+           'plot_morris_results', 'plot_morris_convergence',
+           'plot_fast_results', 'plot_sobol_results')
 
 from collections.abc import Iterable
 import numpy as np
@@ -41,8 +43,6 @@ from SALib.plotting import morris as sa_plt_morris
 from biosteam.plots import plot_spearman
 from .utils import time_printer
 
-isinstance = isinstance
-getattr = getattr
 var_indices = bst.evaluation._model.var_indices
 indices_to_multiindex = bst.evaluation._model.indices_to_multiindex
 
@@ -140,7 +140,7 @@ def get_correlations(model, input_x=None, input_y=None,
         - "omit": drop the pair from analysis.
     file : str
         If provided, the results will be saved as an Excel file.
-    kwargs
+    **kwargs : dict
         Other kwargs that will be passed to ``scipy``.
 
     Returns
@@ -227,7 +227,6 @@ def define_inputs(model):
     return model.problem()
 
 
-# TODO: incorporate sampling from different distributions
 def generate_samples(inputs, kind, N, seed=None, **kwargs):
     '''
     Generate samples for sensitivity analysis using ``SALib``.
@@ -248,7 +247,7 @@ def generate_samples(inputs, kind, N, seed=None, **kwargs):
         The number of samples or trajectories (Morris).
     seed : int
         Seed to generate random samples.
-    kwargs
+    **kwargs : dict
         Other kwargs that will be passed to ``SALib``.
 
     Returns
@@ -273,7 +272,9 @@ def generate_samples(inputs, kind, N, seed=None, **kwargs):
     elif lower == 'rbd':
         return rbd_sampler.sample(inputs, N=N, seed=seed, **kwargs)
     elif lower == 'sobol':
-        return sobol_sampler.sample(inputs, N=N, seed=seed, **kwargs)
+        if seed:
+            raise ValueError('Cannot set seed for Sobol analysis.')
+        return sobol_sampler.sample(inputs, N=N, **kwargs)
     else:
         raise ValueError('kind can only be "FAST", "RBD", "Morris", or "Sobol", ' \
                          f'not "{kind}".')
@@ -313,7 +314,7 @@ def morris_analysis(model, inputs, metrics=None, nan_policy='propagate',
         Whether to show simulation time in the console.
     file : str
         If provided, the results will be saved as an Excel file.
-    kwargs
+    **kwargs : dict
         Other kwargs that will be passed to ``SALib``.
 
     Returns
@@ -348,7 +349,10 @@ def morris_analysis(model, inputs, metrics=None, nan_policy='propagate',
         si = morris.analyze(inputs, param_val.to_numpy(), results.to_numpy(),
                             conf_level=conf_level, print_to_console=print_to_console,
                             **kwargs)
-        morris_dct[metric.name] = si.to_df()
+        df = si.to_df()
+        df.reset_index(inplace=True)
+        df.rename(columns={'index':'parameter'}, inplace=True)
+        morris_dct[metric.name] = df
 
     if file:
         writer = pd.ExcelWriter(file)
@@ -401,7 +405,7 @@ def morris_till_convergence(model, inputs, metrics=None,
         Whether to show simulation time in the console.
     file : str
         If provided, the results will be saved as an Excel file.
-    kwargs
+    **kwargs : dict
         Other kwargs that will be passed to ``SALib``.
 
     Examples
@@ -425,15 +429,16 @@ def morris_till_convergence(model, inputs, metrics=None,
     cum_model.evaluate()
     cum_dct = dict(mu_star={}, mu_star_conf={})
     metrics = _update_input(metrics, model.metrics)
+    kwargs['print_time'] = kwargs.get('print_time') or False
     temp_dct = morris_analysis(model=cum_model, inputs=inputs, metrics=metrics,
                                nan_policy=nan_policy, conf_level=conf_level,
-                               print_to_console=print_to_console, **kwargs)
+                               print_to_console=print_to_console,**kwargs)
     for m in metrics:
         for idx in ('mu_star', 'mu_star_conf'):
             data0 = getattr(temp_dct[m.name], idx)
-            df = pd.DataFrame(columns=data0.index, index=(2,))
+            df = pd.DataFrame(columns=temp_dct[m.name].parameter, index=(2,))
             df.index.name = idx
-            df.loc[2] = data0.copy()
+            df.loc[2] = data0.values
             cum_dct[idx][m.name] = df
 
     for n in range(2, N_max):
@@ -445,21 +450,22 @@ def morris_till_convergence(model, inputs, metrics=None,
         temp_dct = morris_analysis(model=cum_model, inputs=inputs, metrics=metrics,
                                    nan_policy=nan_policy, conf_level=conf_level,
                                    print_to_console=print_to_console, **kwargs)
+
         all_converged = True
         for m in metrics:
             mu_star = temp_dct[m.name].mu_star
             mu_star_conf = temp_dct[m.name].mu_star_conf
-            cum_dct['mu_star'][m.name].loc[n+1] = mu_star
-            cum_dct['mu_star_conf'][m.name].loc[n+1] = mu_star_conf
+            cum_dct['mu_star'][m.name].loc[n+1] = mu_star.values
+            cum_dct['mu_star_conf'][m.name].loc[n+1] = mu_star_conf.values
 
             converged = False if (mu_star_conf/mu_star.max()>threshold).any() else True
             all_converged = all_converged & converged
 
         if all_converged:
-            print(f'mu_star converges at {n} trajectories.')
+            print(f'mu_star converges at # {n+1} trajectories.')
             break
         elif n == N_max-1:
-            print(f'mu_star has not converged with {n} trajectories.')
+            print(f'mu_star has not converged within {n+1} trajectories.')
 
     if file:
         writer = pd.ExcelWriter(file)
@@ -510,7 +516,7 @@ def fast_analysis(model, inputs, kind, metrics=None, nan_policy='propagate',
         Whether to show simulation time in the console.
     file : str
         If provided, the results will be saved as an Excel file.
-    kwargs
+    **kwargs : dict
         Other kwargs that will be passed to ``SALib``.
 
     Returns
@@ -604,7 +610,7 @@ def sobol_analysis(model, inputs, metrics=None, nan_policy='propagate',
         Whether to show simulation time in the console.
     file : str
         If provided, the results will be saved as an Excel file.
-    kwargs
+    **kwargs : dict
         Other kwargs that will be passed to ``SALib``.
 
     Returns
@@ -751,29 +757,18 @@ def plot_uncertainties(model, x_axis=(), y_axis=(), kind='box',
     x_df = y_df = None
     sns_df = pd.DataFrame(columns=('x_group', 'x_data', 'y_group', 'y_data'))
 
-    dfs = []
     if not y_axis: # no data provided or only x, 1D, horizontal
         x_axis = _update_input(x_axis, model.metrics)
         x_df = df[[i.name for i in x_axis]]
-
-        for x in x_df.columns:
-            temp_df = sns_df.copy()
-            temp_df['x_data'] = x_df[x]
-            temp_df['x_group'] = x
-            dfs.append(temp_df)
-
-        sns_df = pd.concat(dfs)
+        temp_df = x_df.stack(dropna=False).reset_index(name='x_data')
+        sns_df['x_data'] = temp_df['x_data']
+        sns_df['x_group'] = temp_df['level_1']
 
     elif not x_axis: # only y, 1D, vertical
         y_df = df[[i.name for i in y_axis]]
-
-        for y in y_df.columns:
-            temp_df = sns_df.copy()
-            temp_df['y_data'] = y_df[y]
-            temp_df['y_group'] = y
-            dfs.append(temp_df)
-
-        sns_df = pd.concat(dfs)
+        temp_df = y_df.stack(dropna=False).reset_index(name='y_data')
+        sns_df['y_data'] = temp_df['y_data']
+        sns_df['y_group'] = temp_df['level_1']
 
     else: # x and y, 2D
         len_x = len_y = 1
@@ -794,7 +789,7 @@ def plot_uncertainties(model, x_axis=(), y_axis=(), kind='box',
     sns.set_theme(style='ticks')
     if not twoD: # 1D plot
         if kind_lower == 'box':
-            center_kws.setdefault('dodge', False)
+            center_kws['dodge'] = center_kws.get('dodge') or False
 
             if x_df is not None: # horizontal box plot
                 ax = sns.boxplot(data=sns_df, x='x_group', y='x_data', hue='x_group',
@@ -835,8 +830,15 @@ def plot_uncertainties(model, x_axis=(), y_axis=(), kind='box',
         g = sns.JointGrid(data=sns_df, x=x_name, y=y_name)
 
         kind_split = kind_lower.split('-')
+        if len(kind_split) == 1:
+            raise ValueError(f'Data input implies 2D-plot, but input `kind` is "{kind}", '
+                             f'implies 1D plot, maybe you want "{kind}-{kind}"?')
+
         if kind_split[0] in ('hist', 'kde'):
             func = getattr(sns, f'{kind_split[0]}plot')
+            #!!! For some reason `seaborn` might raise an error with "dodge"
+            try: center_kws.pop('dodge')
+            except: pass
             g.plot_joint(func, **center_kws)
         else:
             raise ValueError(f'The provided kind "{kind}" is not valid for 2D plot.')
@@ -897,7 +899,6 @@ def _plot_corr_bubble(corr_df, ratio, **kwargs):
 
     g.ax.grid(True, which='major', color='k',linestyle='--', linewidth=0.7)
     g.tight_layout()
-
     return g
 
 
@@ -914,10 +915,10 @@ def plot_correlations(result_df, parameters=(), metrics=(), top=None,
         correlation indices.
     parameters : :class:`biosteam.Parameter`
         Metric(s) of interest for the plot, will be default to all parameters
-        included in ``corr_dct`` if not provided.
+        included in `result_df` if not provided.
     metrics : :class:`biosteam.Metric`
         Metric(s) of interest for the plot, will be default to all metrics
-        included in ``corr_dct`` if not provided.
+        included in `result_df` if not provided.
     top : int
         Plot the top X parameters with the highest absolute correlation indices,
         this is only applicable for the case of just one metric.
@@ -926,7 +927,7 @@ def plot_correlations(result_df, parameters=(), metrics=(), top=None,
     close_fig : bool
         Whether to close the figure
         (if not close, new figure will be overlaid on the current figure).
-    kwargs
+    **kwargs: dict
         Other kwargs that will be passed to :func:`seaborn.relplot`.
 
     Returns
@@ -967,15 +968,10 @@ def plot_correlations(result_df, parameters=(), metrics=(), top=None,
         return _save_fig_return(fig, ax, file, close_fig)
 
     else: # multiple metrics, bubble plot
-        corr_df = pd.DataFrame()
-
-        for m in df.columns:
-            temp_df = pd.DataFrame(columns=('parameter', 'metric', 'correlation', 'size'))
-            temp_df['parameter'] = df.index
-            temp_df['metric'] = [m]*temp_df.shape[0]
-            temp_df['correlation'] = df[m].values
-            temp_df['size'] = np.abs(df[m].values)
-            corr_df = pd.concat((corr_df, temp_df))
+        corr_df = df.stack(dropna=False).reset_index()
+        corr_df.rename(columns={'level_0': 'parameter', 'level_1': 'metric',
+                                0: 'correlation'}, inplace=True)
+        corr_df['size'] = corr_df['correlation'].abs()
 
         g = _plot_corr_bubble(corr_df, len(metric_names)/len(param_names), **kwargs)
 
@@ -987,10 +983,10 @@ def plot_correlations(result_df, parameters=(), metrics=(), top=None,
 # Plot Morris analysis results
 # =============================================================================
 
-def plot_morris_results(morris_dct, metric, kind='scatter',
-                        x_axis='mu_star',
+def plot_morris_results(morris_dct, metric, kind='scatter', ax=None,
+                        x_axis='mu_star', plot_lines=True,
                         k1=0.1, k2=0.5, k3=1, label_kind='number',
-                        file='', close_fig=True, **kwargs):
+                        color='k', file='', close_fig=True, **kwargs):
     '''
     Visualize the results from Morris One-at-A-Time analysis as either scatter
     or bar plots.
@@ -1006,6 +1002,8 @@ def plot_morris_results(morris_dct, metric, kind='scatter',
         The metric of interest for the plot.
     kind : str
         Either "scatter" (:math:`{\sigma}`) vs. :math:`{\mu^*}` or "bar" (:math:`{\mu^*}` with confidence interval) plot.
+    ax : :class:`matplotlib.AxesSubplot`
+        If provided, the figure will be plotted on this axis.
     x_axis : str
         X-axis parameter, should be either "mu_star" (the commonly used one) or "mu".
     k1 : float
@@ -1018,14 +1016,17 @@ def plot_morris_results(morris_dct, metric, kind='scatter',
         The slope to differentiate non-linear and/or non-monotonic (above the line)
         and almost monotonic (below the line).
     label_kind : str
-        How to label the points, can be either "number" (use index number of the result table)
-        of "name" (use index name of the result table).
+        How to label the points in the scatter plot,
+        can be "number" (use index number of the result table),
+        "name" (use index name of the result table), or None (no labels).
+    color : str or RGBs
+        Plot color.
     file : str
         If provided, the generated figure will be saved as a png file.
     close_fig : bool
         Whether to close the figure
         (if not close, new figure will be overlaid on the current figure).
-    kwargs
+    **kwargs : dict
         Other kwargs that will be passed to :func:`morris.horizontal_bar_plot` in ``SALib.plotting``.
 
     Returns
@@ -1042,31 +1043,49 @@ def plot_morris_results(morris_dct, metric, kind='scatter',
 
     df = morris_dct[metric.name]
     x_data = getattr(df, x_axis)
+    x_error = df.mu_star_conf if x_axis=='mu_star' else None
     y_data = df.sigma
-    num = len(x_data)
     if label_kind == 'number':
-        labels = range(num)
-    elif label_kind == 'name':
         labels = df.index.values
+    elif label_kind == 'name':
+        labels = df.parameter
     else:
-        raise ValueError(f'label_kind can only be "number" or "name", not "{label_kind}".')
+        labels = []
 
-    ax = plt.subplot()
+    ax = ax if ax is not None else plt.subplot()
     sns.set_theme(style='ticks')
 
     if kind == 'scatter':
-        ax.scatter(x_data, y_data, color='k')
+        if x_error is not None:
+            ax.errorbar(x=x_data, y=y_data, xerr=x_error, fmt='.',
+                        color=color)
+        else:
+            ax.scatter(x_data, y_data, color=color)
         for x, y, label in zip(x_data, y_data, labels):
             ax.annotate(label, (x, y), xytext=(10, 10), textcoords='offset points',
                         ha='center')
         x_range = np.arange(-1, np.ceil(ax.get_xlim()[1])+1)
-        line1, = ax.plot(x_range, k1*x_range, color='black', linestyle='-.')
-        line2, = ax.plot(x_range, k2*x_range, color='black', linestyle='--')
-        line3, = ax.plot(x_range, k3*x_range, color='black', linestyle='-')
-        ax.legend((line3, line2, line1), (r'$\sigma/\mu^*$'+f'={k3}',
-                                          r'$\sigma/\mu^*$'+f'={k2}',
-                                          r'$\sigma/\mu^*$'+f'={k1}'),
-                  loc='best')
+
+        lines, legends = [], []
+        line_color = kwargs.get('line_color') or color
+        if k3:
+            line3, = ax.plot(x_range, k3*x_range, color=line_color, linestyle='-')
+            lines.append(line3)
+            legends.append(r'$\sigma/\mu^*$'+f'={k3}')
+
+        if k2:
+            line2, = ax.plot(x_range, k2*x_range, color=line_color, linestyle='--')
+            lines.append(line2)
+            legends.append(r'$\sigma/\mu^*$'+f'={k2}')
+
+        if k1:
+            line1, = ax.plot(x_range, k1*x_range, color=line_color, linestyle='-.')
+            lines.append(line1)
+            legends.append(r'$\sigma/\mu^*$'+f'={k1}')
+
+        if len(lines) > 0:
+            ax.legend(lines, legends, loc='best')
+
         if x_axis == 'mu_star':
             ax.set_xlim(0,)
         ax.set_ylim(0,)
@@ -1075,13 +1094,13 @@ def plot_morris_results(morris_dct, metric, kind='scatter',
         fig = ax.figure
 
     elif kind == 'bar':
+        kwargs['color'] = color
         if x_axis == 'mu':
             raise ValueError('Bar plot can only be made for mu_star, not mu.')
         df = morris_dct[metric.name]
         df['names'] = df.index
         fig = sa_plt_morris.horizontal_bar_plot(ax, df, opts=kwargs)
 
-    # for ax in fig.axes:
     for key in ax.spines.keys():
         ax.spines[key].set(color='k', linewidth=0.5, visible=True)
         ax.grid(False)
@@ -1089,8 +1108,9 @@ def plot_morris_results(morris_dct, metric, kind='scatter',
     return _save_fig_return(fig, ax, file, close_fig)
 
 
-def plot_morris_convergence(result_dct, metric, parameters=(), plot_rank=False,
-                            kind='line', show_error=True, palette='pastel', file='', close_fig=True):
+def plot_morris_convergence(result_dct, metric, parameters=(),
+                            plot_rank=False, kind='line', ax=None, show_error=True,
+                            palette='pastel', file='', close_fig=True):
     '''
     Plot the evolution of :math:`{\mu^*}` or its rank with the number of trajectories.
 
@@ -1111,6 +1131,8 @@ def plot_morris_convergence(result_dct, metric, parameters=(), plot_rank=False,
             If plot_rank is True, error bars or bands will not be included.
     kind : str
         Can be either 'line' or 'scatter'.
+    ax : :class:`matplotlib.AxesSubplot`
+        If provided, the figure will be plotted on this axis.
     show_error : bool
         Whether to include the confidence interval in the plot,
         will be bars for scatter plot and bands for line plot.
@@ -1134,7 +1156,7 @@ def plot_morris_convergence(result_dct, metric, parameters=(), plot_rank=False,
     `QSDsan.stats <https://qsdsan.readthedocs.io/en/latest/stats.html>`_
     '''
 
-    ax = plt.subplot()
+    ax = ax if ax is not None else plt.subplot()
     df = result_dct['mu_star'][metric.name].copy().astype('float64')
     conf_df = result_dct['mu_star_conf'][metric.name].copy().astype('float64')
 
@@ -1229,8 +1251,8 @@ def _plot_heatmap(hmap_df, ax=None, annot=False, diagonal='', sts1_df=None,
     return ax
 
 
-def plot_fast_results(result_dct, metric, parameters=(), error_bar=True,
-                      file='', close_fig=True):
+def plot_fast_results(result_dct, metric, parameters=(),
+                      ax=None, error_bar=True, file='', close_fig=True):
     '''
     Visualize the results from FAST or RBD-FAST analysis as a bar plot.
 
@@ -1244,6 +1266,8 @@ def plot_fast_results(result_dct, metric, parameters=(), error_bar=True,
         Single or a iterable of model parameters whose :math:`{\mu^*}` will be
         included in the plot.
         Will be set to all parameters in retult_dct will be used if not provided.
+    ax : :class:`matplotlib.AxesSubplot`
+        If provided, the figure will be plotted on this axis.
     error_bar : bool
         Whether to include the confidence interval as error bars in the plot.
     file : str
@@ -1271,12 +1295,13 @@ def plot_fast_results(result_dct, metric, parameters=(), error_bar=True,
     df = result_dct[metric.name]
     kind = 'STS1' if 'ST' in df.columns else 'S1'
 
-    ax = _plot_bar(kind, df, error_bar)
+    ax = _plot_bar(kind, df, error_bar, ax=ax)
 
     return _save_fig_return(ax.figure, ax, file, close_fig)
 
 
-def plot_sobol_results(result_dct, metric, parameters=(), kind='all',
+def plot_sobol_results(result_dct, metric, ax=None,
+                       parameters=(), kind='all',
                        annotate_heatmap=False, plot_in_diagonal='',
                        error_bar=True, file='', close_fig=True):
     '''
