@@ -21,6 +21,7 @@ from . import (
     colors,
     decorators,
     # descriptors, # currently not in use
+    evaluation,
     getters,
     formatting,
     loading,
@@ -29,10 +30,13 @@ from . import (
     units_of_measure,
     )
 
+from biosteam.utils import NotImplementedMethod
+
 from .cod import *
 from .colors import *
 from .decorators import *
 # from .descriptors import *
+from .evaluation import *
 from .getters import *
 from .formatting import *
 from .loading import *
@@ -46,12 +50,14 @@ __all__ = (
     *colors.__all__,
     *decorators.__all__,
     # *descriptors.__all__,
+    *evaluation.__all__,
     *getters.__all__,
     *formatting.__all__,
     *loading.__all__,
     *parse.__all__,
     *setters.__all__,
     *units_of_measure.__all__,
+    'NotImplementedMethod',
     )
 
 
